"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.examples.exampleskeleton

This file provides the basic skeleton for other, simple, examples.
"""
from __future__ import division
# Magic so that examples work if trying from an extracted archive
try:
    import pygl3display as pyg3
except ImportError:
    import sys, os.path
    sys.path.append(os.path.join('..', '..'))
    import pygl3display as pyg3

import pygame
#from pygl3display import *
pyg3.configure('pygame')
from pygame.locals import KEYDOWN, K_ESCAPE, QUIT 

class ExampleSkeleton(object):
    """A skeleton app used in examples"""
    def __init__(self, name, resolution):
        """Initialise the test app data"""
        self.name = name
        self.screenResolution = resolution
        
    def init(self):
        """Override for initialisation of the window"""
        pass
        
    def loop(self, dt):
        """Override for updating game state per clock"""
        pass
        
    def eventHandler(self, event):
        """Override to handle game events"""
        pass
        
    def run(self, ticks=None):
        """Run the example"""
        pyg3.GLCommon.initEnvironment(self.screenResolution)
        pygame.display.set_caption(self.name)
        self.init()
        self.clock = pygame.time.Clock()
        self.going = True
        while self.going:
            for event in pygame.event.get():
                if event.type == QUIT or (event.type == KEYDOWN 
                   and event.key == K_ESCAPE):
                    self.going = False
                else:
                    self.eventHandler(event)
            dt = self.clock.tick(60)
            self.loop(dt)
            pyg3.GLCommon.doTasks()
            if ticks is not None:
                ticks -= 1
                if ticks == 0:
                    self.going = False
        pyg3.GLCommon.quit()
