"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.examples.linesexample

This file provides an example of most of the uses of PyGL3Display liness.
Also provides the major test case.
"""

from exampleskeleton import ExampleSkeleton, pyg3
import math

class LineApp(ExampleSkeleton):
    """Showcase and test lines"""
    def __init__(self):
        """Initialise the test app data"""
        super(LineApp, self).__init__('Lines', (70, 70))
        
    def init(self):
        """Create example content after context created"""
        red = (255, 0, 0, 255)
        green = (0, 255, 0, 255)
        blue = (0, 0, 255, 255)
        square = [(-10, 10), (10, 10), (10, -10), (-10, -10)]
        self.lines1 = pyg3.Lines(
                vertices=[(10, 10), (10, 30), (30, 30), (30, 10)], width=4)
        self.lines2 = pyg3.Lines(vertices=square, pos=(45, 20),
                colors=[red, green, blue, green], closed=True)
        self.lines3 = pyg3.Lines(vertices=square, pos=(20, 45),
                color=blue, layer=5, width=4)
        self.lines1.setLayer(3)
        self.lines1.setWidth(3)
        self.lg5term = 35
        self.lines4 = pyg3.Lines(
                vertices=[(35, 35), (35, 55), (55, 55), (55, 35)])
        del self.lines4  
        self.lines5 = pyg3.Lines(
                vertices=[(0, 0), (35, 35)], color=green, layer=4)
        self.lg5term = 35
        self.lgAdd = 1
        self.lg2rot = 0.0
        self.lg2add = 0.1
        self.camera = pyg3.TwoDVertexCamera()
        self.lines3.setCamera(0, self.camera) 
        
    def loop(self, dt):
        """Move some of the lines around"""
        self.lg5term += 1 * self.lgAdd
        if self.lg5term in (0, 50): 
            self.lgAdd *= -1 
        self.lg2rot += self.lg2add
        if self.lg2rot > 2*math.pi: 
            self.lg2rot -= 2 * math.pi
        self.lines2.setRotation(self.lg2rot)
        self.lines5.setVertex(1, (35, self.lg5term))
        self.camera.setCameraPosition((self.lg5term, 0))

if __name__ == '__main__':
    LineApp().run()
