"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.examples.rendertotexture

Provides an example of render to texture support.
"""

from exampleskeleton import pyg3
from showcase import ShowCaseApp
# TODO: Why explicit import? Should be in pygl3displays namespace, hopefully
from pygl3display.sprites.texconverter import Converter 
  
class RenderToTextureApp(ShowCaseApp):
    """Renders the Showcase demo to a texture, then renders
    the texture"""
    def __init__(self):
        """Initialise the test app data"""
        super(RenderToTextureApp, self).__init__('RenderToTexture')
        self.count = 0
        self.total = 0
        
    def init(self):
        """Initialise the example; all the showcase items here are being 
        rendered onto a texture target"""
        self.target = pyg3.TextureTarget(1, 128)
        super(RenderToTextureApp, self).init(self.target)
        

    def loop(self, dt):
        """Render the texture target, then make a sprite based on it that will
        be drawn to the main screen"""
        self.count += dt
        pyg3.GLCommon.doTasks(self.target)
        self.spriteImage = Converter.textureAsSprite(self.target.textures[0])
        

if __name__ == '__main__':
    RenderToTextureApp().run()
