"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.examples.showcase

Draws a single primitive of each shader type in PyGL3Display. This is 
to demonstrate what is available and ensure that all primitives work together.
"""
from exampleskeleton import ExampleSkeleton, pyg3
import math

class ShowCaseApp(ExampleSkeleton):
    """A showcase of things PyGL3Display can do"""
    def __init__(self, title='Showcase'):
        """Initialise the test app data"""
        super(ShowCaseApp, self).__init__(title, (70, 70))
        
    def init(self, target=None):
        """Initialise the example"""
        red = (255, 0, 0, 255)
        green = (0, 255, 0, 255)
        tri = [(-10, 10), (10, 10), (10, -10)]
        self.ellipse = pyg3.Ellipse(dx = 5, dy = 15, pos = (50, 20), 
            rotation = math.pi / 4, color = green, target = target)
        self.lines = pyg3.Lines(
            vertices = [(10, 10), (10, 30), (30, 30), (30, 10)], 
            target = target)
        pyg3.SpriteCommon.setDefaultFilter('nearest')
        self.block = pyg3.loadImage('data/baseblock.png', wrapped = True)
        self.sprite = pyg3.Sprite(image = self.block, position = (10, 35), 
            tiling = (3, 3), target = target)
        self.tri1 = pyg3.Triangles(vertices = tri, pos = (50, 50), 
            color = red, target = target)


if __name__ == '__main__':
    ShowCaseApp().run()
