"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.gl

Provides the GL Binding of PyGL3Display. This can either be OpenGL or CFFI
implementation"""

import os
backend = None
if os.name in ('posix',):
    try:
        from pygl3display.gl.cffigl import *
        backend = 'cffigl'
    except ImportError:
        import sys
        sys.stderr.write('Could not import cffigl backend; falling back to pyopengl')

if backend is None:
    from pygl3display.gl.pyopengl import *
    from pygl3display.gl.ctypesdata import *
    backend = 'pyopengl'

class TexUnitsDict(dict):
    """Maps integers to available texture units"""
    def __missing__(self, key):
        """Lazily create the map between integer i and GL_TEXTUREi"""
        if isinstance(key, int) and 0 <= key < GL_MAX_TEXTURE_COORDS:
            tuname = 'GL_TEXTURE%i' % key
            self[key] = globals()[tuname]
            return self[key]
        else:
            raise KeyError('%s is not a valid Texture Unit' % str(key))
        
class ColorAttachmentDict(dict):
    def __missing__(self, key):
        names = [key, key + 'EXT', key + '_EXT']
        lookup = globals()
        for name in names:
            if name in lookup:
                return lookup[name]
        raise Exception('Couldn\'t find %s' % key)
