//PyGL3Display
//(C) David Griffin (2010-present)
//"Habilain" is a pseudonym used by David Griffin in a number of places.
//
//A high performance Pygame OpenGL library, capable of handling 1000s of sprites
//
//Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware
//
//Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
//licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
//found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
//http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
//to be bound by at least one of these licenses.
//
//Other licenses, including commercial/proprietary licenses, can be arranged by 
//contacting me via e-mail at habilain@gmail.com. If you do not have in physical 
//writing an alternative license agreement from me, you must use one of the 
//licenses specified above when releasing work which makes use of this library.
//
//cffigl.h
//
//Part of the experimental cffi OpenGL binding

typedef unsigned int	GLenum;
typedef unsigned char	GLboolean;
typedef char	GLchar;
typedef unsigned int	GLbitfield;
typedef void		GLvoid;
typedef signed char	GLbyte;		/* 1-byte signed */
typedef short		GLshort;	/* 2-byte signed */
typedef int		GLint;		/* 4-byte signed */
typedef unsigned char	GLubyte;	/* 1-byte unsigned */
typedef unsigned short	GLushort;	/* 2-byte unsigned */
typedef unsigned int	GLuint;		/* 4-byte unsigned */
typedef int		GLsizei;	/* 4-byte signed */
typedef float		GLfloat;	/* single precision float */
typedef float		GLclampf;	/* single precision float in [0,1] */
typedef double		GLdouble;	/* double precision float */
typedef double		GLclampd;	/* double precision float in [0,1] */
// This needs checking
typedef int GLsizeiptr;
typedef int GLintptr;

typedef void (* GLFWwindowsizefun)(int,int);
typedef int  (* GLFWwindowclosefun)(void);
typedef void (* GLFWwindowrefreshfun)(void);
typedef void (* GLFWmousebuttonfun)(int,int);
typedef void (* GLFWmouseposfun)(int,int);
typedef void (* GLFWmousewheelfun)(int);
typedef void (* GLFWkeyfun)(int,int);
typedef void (* GLFWcharfun)(int,int);
typedef void (* GLFWthreadfun)(void *);

// GL stuff
const GLubyte* glGetString(GLenum name);
const GLubyte* glGetStringi(GLenum name, GLuint index);
void glClearColor(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
void glBindBuffer(GLenum  target,  GLuint  buffer);
void glBindVertexArray(GLuint array);
void glBufferData(GLenum  target,  GLsizeiptr  size,  const GLvoid *  data,  GLenum  usage);
void glEnableVertexAttribArray(GLuint  index);
void glDisableVertexAttribArray(GLuint  index);
void glClear(GLbitfield mask);
void glVertexAttribPointer(GLuint  index,  GLint  size,  GLenum  type,  GLboolean  normalized,  GLsizei  stride,  const GLvoid *  pointer);
void glDrawArrays(GLenum  mode,  GLint  first,  GLsizei  count);
GLuint glCreateShader(GLenum shaderType);
void glShaderSource(GLuint shader, GLsizei count, const GLchar ** string, const GLint * length);
void glCompileShader(GLuint shader);
void glGetShaderiv(GLuint shader,  GLenum pname,  GLint *params);
void glGetShaderInfoLog(GLuint  shader,  GLsizei  maxLength,  GLsizei * length,  GLchar * infoLog);
GLuint glCreateProgram(void);
void glAttachShader(GLuint program,  GLuint shader);
void glLinkProgram(GLuint program);
void glGetProgramiv(GLuint program,  GLenum pname,  GLint *params);
void glGetProgramInfoLog(GLuint program,  GLsizei maxLength,  GLsizei *length,  GLchar *infoLog);
void glDeleteShader(GLuint shader);
GLint glGetAttribLocation(GLuint  program,  const GLchar * name);
void glUseProgram(GLuint program);
GLint glGetUniformLocation(GLuint  program,  const GLchar * name);
GLenum glGetError( void);
void glEnable(GLenum cap);
void glDisable(GLenum cap);
void glDepthFunc(GLenum func);
void glBindTexture(GLenum  target,  GLuint  texture);
void glActiveTexture(GLenum texture);
void glUniform1f(GLint location,  GLfloat v0);
void glUniform2f(GLint location,  GLfloat v0,  GLfloat v1);
void glUniform3f(GLint location,  GLfloat v0,  GLfloat v1,  GLfloat v2);
void glUniform4f(GLint location,  GLfloat v0,  GLfloat v1,  GLfloat v2,  GLfloat v3);
void glUniform1i(GLint location,  GLint v0);
void glUniform2i(GLint location,  GLint v0,  GLint v1);
void glUniform3i(GLint location,  GLint v0,  GLint v1,  GLint v2);
void glUniform4i(GLint location,  GLint v0,  GLint v1,  GLint v2,  GLint v3);
void glTexParameterf(GLenum  target,  GLenum  pname,  GLfloat  param);
void glTexParameteri(GLenum  target,  GLenum  pname,  GLint  param);
void glFlush( void);
void glBufferSubData(GLenum  target,  GLintptr  offset,  GLsizeiptr  size,  const GLvoid *  data);
void glHint(GLenum target, GLenum mode);
void glDeleteProgram(GLuint program);
void glViewport(GLint  x,  GLint  y,  GLsizei  width,  GLsizei  height);
void glLineWidth(GLfloat width);
void glDeleteTextures(GLsizei  n,  const GLuint *  textures);
void glBlendFunc(GLenum  sfactor,  GLenum  dfactor);
void glBlendFunci(GLuint  buf,  GLenum  sfactor,  GLenum  dfactor);
void glTexImage1D(GLenum  target,  GLint  level,  GLint  internalFormat,  GLsizei  width,  GLint  border,  GLenum  format,  GLenum  type,  const GLvoid *  data);
void glTexImage2D(GLenum  target,  GLint  level,  GLint  internalFormat,  GLsizei  width,  GLsizei  height,  GLint  border,  GLenum  format,  GLenum  type,  const GLvoid *  data);
void glTexImage3D(GLenum  target,  GLint  level,  GLint  internalFormat,  GLsizei  width,  GLsizei  height,  GLsizei  depth,  GLint  border,  GLenum  format,  GLenum  type,  const GLvoid *  data);
void glTexSubImage1D(GLenum  target,  GLint  level,  GLint  xoffset,  GLsizei  width,  GLenum  format,  GLenum  type,  const GLvoid *  data);
void glTexSubImage2D(GLenum  target,  GLint  level,  GLint  xoffset,  GLint  yoffset,  GLsizei  width,  GLsizei  height,  GLenum  format,  GLenum  type,  const GLvoid *  data);
void glTexSubImage3D(GLenum  target,  GLint  level,  GLint  xoffset,  GLint  yoffset,  GLint  zoffset,  GLsizei  width,  GLsizei  height,  GLsizei  depth,  GLenum  format,  GLenum  type,  const GLvoid *  data);
void glGetTexImage(GLenum  target,  GLint  level,  GLenum  format,  GLenum  type,  GLvoid *  img);
void glReadPixels(GLint  x,  GLint  y,  GLsizei  width,  GLsizei  height,  GLenum  format,  GLenum  type,  GLvoid *  data);
void glGetTexLevelParameterfv(GLenum  target,  GLint  level,  GLenum  pname,  GLfloat *  params);
void glGetTexLevelParameteriv(GLenum  target,  GLint  level,  GLenum  pname,  GLint *  params);
void glReadBuffer(GLenum mode);
void glDrawBuffers(GLsizei  n,  const GLenum * bufs);
void glDrawElements(GLenum  mode,  GLsizei  count,  GLenum  type,  const GLvoid *  indices);
void glGetIntegerv(GLenum name, GLint * dataType);
void glGetBooleanv(GLenum name, GLboolean * dataType);
void glGetFloatv(GLenum name, GLfloat * dataType);
void glGetDoublev(GLenum name, GLdouble * dataType);
void glGenBuffers(GLsizei  n,  GLuint *  buffers);
void glGenVertexArrays(GLsizei  n,  GLuint *  buffers);
void glGenTextures(GLsizei  n,  GLuint *  buffers);
void glUniform1fv(GLint location,  GLsizei count, const GLfloat *value);
void glUniform2fv(GLint location,  GLsizei count, const GLfloat *value);
void glUniform3fv(GLint location,  GLsizei count, const GLfloat *value);
void glUniform4fv(GLint location,  GLsizei count, const GLfloat *value);
void glUniform1iv(GLint location,  GLsizei count, const GLint *value);
void glUniform2iv(GLint location,  GLsizei count, const GLint *value);
void glUniform3iv(GLint location,  GLsizei count, const GLint *value);
void glUniform4iv(GLint location,  GLsizei count, const GLint *value);
void glUniformMatrix2fv(GLint location,  GLsizei count,  GLboolean transpose,  const GLfloat *value);
void glUniformMatrix3fv(GLint location,  GLsizei count,  GLboolean transpose,  const GLfloat *value);
void glUniformMatrix4fv(GLint location,  GLsizei count,  GLboolean transpose,  const GLfloat *value);
GLboolean glIsTexture(GLuint texture);

// gl3.1 framebuffer support
// Note: this is not core in 2.1, but may work anyway. For now,
// just go with these and if necessary later define how to work with
// the EXT version.
// As to why it doesn't work in PyOpenGL.. who knows?
void glGenFramebuffers(GLsizei  n,  GLuint *  buffers);
GLenum glCheckFramebufferStatus(GLenum  target);
void glFramebufferTexture(GLenum  target,  GLenum  attachment,  GLuint  texture,  GLint  level);
void glFramebufferTexture1D(GLenum  target,  GLenum  attachment,  GLenum  textarget,  GLuint  texture,  GLint  level);
void glFramebufferTexture2D(GLenum  target,  GLenum  attachment,  GLenum  textarget,  GLuint  texture,  GLint  level);
void glFramebufferTexture3D(GLenum  target,  GLenum  attachment,  GLenum  textarget,  GLuint  texture,  GLint  level,  GLint  layer);
void glBindFramebuffer(GLenum  target,  GLuint  framebuffer);

// gl2.1 framebuffer EXT support
/*
void glGenFramebuffersEXT(GLsizei  n,  GLuint *  buffers);
GLenum glCheckFramebufferStatusEXT(GLenum  target);
void glFramebufferTextureEXT(GLenum  target,  GLenum  attachment,  GLuint  texture,  GLint  level);
void glFramebufferTexture1DEXT(GLenum  target,  GLenum  attachment,  GLenum  textarget,  GLuint  texture,  GLint  level);
void glFramebufferTexture2DEXT(GLenum  target,  GLenum  attachment,  GLenum  textarget,  GLuint  texture,  GLint  level);
void glFramebufferTexture3DEXT(GLenum  target,  GLenum  attachment,  GLenum  textarget,  GLuint  texture,  GLint  level,  GLint  layer);
void glBindFramebufferEXT(GLenum  target,  GLuint  framebuffer);
*/
// glfw stuff
int glfwInit( void );
void glfwTerminate( void );
void glfwGetVersion( int *major, int *minor, int *rev );
int glfwOpenWindow( int width, int height, int redbits,
int greenbits, int bluebits, int alphabits, int depthbits,
int stencilbits, int mode );
void glfwOpenWindowHint( int target, int hint );
void glfwCloseWindow( void );
void glfwSetWindowCloseCallback( GLFWwindowclosefun cbfun );
void glfwSetWindowTitle( const char *title );
void glfwSetWindowSize( int width, int height );
void glfwSetWindowPos( int x, int y );
void glfwGetWindowSize( int *width, int *height );
void glfwSetWindowSizeCallback( GLFWwindowsizefun cbfun );
void glfwIconifyWindow( void );
void glfwRestoreWindow( void );
int glfwGetWindowParam( int param );
void glfwSwapBuffers( void );
void glfwSwapInterval( int interval );
void glfwSetWindowRefreshCallback( GLFWwindowrefreshfun cbfun );
//int glfwGetVideoModes( GLFWvidmode *list, int maxcount );
//void glfwGetDesktopMode( GLFWvidmode *mode );
void glfwPollEvents( void );
void glfwWaitEvents( void );
int glfwGetKey( int key );
int glfwGetMouseButton( int button );
void glfwGetMousePos( int *xpos, int *ypos );
void glfwSetMousePos( int xpos, int ypos );
int glfwGetMouseWheel( void );
void glfwSetMouseWheel( int pos );
void glfwSetKeyCallback( GLFWkeyfun cbfun );
void glfwSetCharCallback( GLFWcharfun cbfun );
void glfwSetMouseButtonCallback( GLFWmousebuttonfun cbfun );
void glfwSetMousePosCallback( GLFWmouseposfun cbfun );
void glfwSetMouseWheelCallback( GLFWmousewheelfun cbfun );
int glfwGetJoystickParam( int joy, int param );
int glfwGetJoystickPos( int joy, float *pos, int numaxes );
int glfwGetJoystickButtons( int joy, unsigned char *buttons,
int numbuttons );
double glfwGetTime( void );
void glfwSetTime( double time );
void glfwSleep( double time );
void glfwEnable( int token );
void glfwDisable( int token );

// glew stuff

int glewInit(void);
