"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance Pygame OpenGL library, capable of handling 1000s of sprites

Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
to be bound by at least one of these licenses.

Other licenses, including commercial/proprietary licenses, can be arranged by 
contacting me via e-mail at habilain@gmail.com. If you do not have in physical 
writing an alternative license agreement from me, you must use one of the 
licenses specified above when releasing work which makes use of this library.

pygl3display.gl.cffigl.cffigl_dev3

Python port of http://www.opengl-tutorial.org/ Example 3"""

from cffigl import *
from cffigl_dev_share import *
import math

vertexShader = """
#version 120

// Input vertex data, different for all executions of this shader.
attribute vec3 vertexPosition_modelspace;

// Values that stay constant for the whole mesh.
uniform mat4 MVP;

void main(){

    // Output position of the vertex, in clip space : MVP * position
    gl_Position =  MVP * vec4(vertexPosition_modelspace,1);

}"""

fragmentShader = """
void main()
{
    // Output color = red 
    gl_FragColor = vec4(1.0,0,0,1.0);

}
"""

gl = GL()
makeWindow(gl, (800, 600), "Test Title")
gl.glfwEnable(gl.GLFW_STICKY_KEYS)
vertexArrayID = gl.glGenVertexArrays()
gl.glBindVertexArray(vertexArrayID)
gl.glClearColor(0.0, 0.0, 0.3, 1.0)
programID = LoadShaders(gl, vertexShader, fragmentShader)
vertexPosition_modelspace_loc = gl.glGetAttribLocation(programID, 'vertexPosition_modelspace')
matrixID = gl.glGetUniformLocation(programID, "MVP");
data = gl.new('GLfloat[]', [-1.0, -1.0, 0.0, 1.0, -1.0, 0.0, 0.0, 1.0, 0.0])
vertexbuffer =  gl.glGenBuffers()
gl.glBindBuffer(gl.GL_ARRAY_BUFFER, vertexbuffer)
gl.glBufferData(gl.GL_ARRAY_BUFFER, gl.ffi.sizeof(data), 
                data, gl.GL_STATIC_DRAW)
projectionMatrix = perspective(math.pi/4.0, 4.0 / 3.0, 0.1, 100.0)
viewMatrix = lookat(vec3(4.0,3.0,3.0), vec3(0.0,0.0,0.0), vec3(0.0,1.0,0.0))                      
mvp = viewMatrix * projectionMatrix 
mvpd = mvp.data()
while (gl.glfwGetKey(gl.GLFW_KEY_ESC) != gl.GLFW_PRESS and 
                    gl.glfwGetWindowParam(gl.GLFW_OPENED)):
    gl.glClear(gl.GL_COLOR_BUFFER_BIT)
    gl.glUseProgram(programID)
    gl.glUniformMatrix4fv(matrixID, 1, gl.GL_FALSE, mvpd);
    gl.glEnableVertexAttribArray(0);
    gl.glBindBuffer(gl.GL_ARRAY_BUFFER, vertexbuffer);
    gl.glVertexAttribPointer(vertexPosition_modelspace_loc, 3, gl.GL_FLOAT, 
                                 gl.GL_FALSE, 0, gl.offsetPointer(0));
    gl.glDrawArrays(gl.GL_TRIANGLES, 0, 3);
    gl.glDisableVertexAttribArray(0);
    gl.glfwSwapBuffers()
gl.glfwTerminate()
