"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance Pygame OpenGL library, capable of handling 1000s of sprites

Requirements: Python, Pygame, PyOpenGL, Open GL 2.0 graphics hardware

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). By using this file you agree
to be bound by at least one of these licenses.

Other licenses, including commercial/proprietary licenses, can be arranged by 
contacting me via e-mail at habilain@gmail.com. If you do not have in physical 
writing an alternative license agreement from me, you must use one of the 
licenses specified above when releasing work which makes use of this library.

pygl3display.gl.cffigl.gl_raw

Provides a raw binding onto GL/other libraries, with support for exporting 
Integer constants. This is currently a work in progress, and *absolutely
will change*"""

from cffi import FFI
import os.path

#glutInts = ['GLUT_DOUBLE', 'GLUT_RGB']

glInts = [
# Color formats / component selection
'GL_BGR', 'GL_BGRA', 'GL_BLUE', 'GL_RED', 'GL_RGB', 'GL_RGBA', 'GL_RGBA32F',
'GL_GREEN', 'GL_RG', 'GL_SRGB', 'GL_S', 'GL_T', 'GL_R', 'GL_Q',
# Data Types
'GL_BYTE', 'GL_SHORT', 'GL_INT', 'GL_UNSIGNED_BYTE', 'GL_UNSIGNED_SHORT', 
'GL_UNSIGNED_INT', 'GL_FLOAT', 'GL_DOUBLE', 
# Singleton values
'GL_TRUE', 'GL_FALSE', 'GL_NONE',
# Texture codes
'GL_PROXY_TEXTURE_1D', 'GL_PROXY_TEXTURE_2D', 'GL_PROXY_TEXTURE_3D',
'GL_TEXTURE_1D', 'GL_TEXTURE_2D', 'GL_TEXTURE_3D',
'GL_TEXTURE_HEIGHT', 'GL_TEXTURE_WIDTH', 'GL_TEXTURE_CUBE_MAP',
# Shader related
'GL_FRAGMENT_SHADER', 'GL_VERTEX_SHADER', 'GL_LINK_STATUS', 
'GL_COMPILE_STATUS', 
# Error Codes
'GL_NO_ERROR', 'GL_INVALID_ENUM', 'GL_INVALID_VALUE', 'GL_OUT_OF_MEMORY',
'GL_INVALID_OPERATION', 'GL_INVALID_FRAMEBUFFER_OPERATION', 
'GL_STACK_UNDERFLOW', 'GL_STACK_OVERFLOW',
# Buffer data types
'GL_STREAM_DRAW', 'GL_STREAM_READ', 'GL_STREAM_COPY', 
'GL_STATIC_DRAW', 'GL_STATIC_READ', 'GL_STATIC_COPY', 
'GL_DYNAMIC_DRAW', 'GL_DYNAMIC_READ', 'GL_DYNAMIC_COPY',
# glGet Arguments
'GL_MAX_COLOR_ATTACHMENTS', 'GL_MAX_TEXTURE_COORDS', 'GL_MAX_TEXTURE_SIZE', 
'GL_MAX_VERTEX_UNIFORM_COMPONENTS', 
# glGetString Arguments
'GL_EXTENSIONS',
# Tests
'GL_LESS',  'GL_EQUAL', 'GL_LEQUAL', 'GL_GREATER', 
'GL_NOTEQUAL', 'GL_GEQUAL', 'GL_NEVER', 'GL_ALWAYS',
# Shapes
'GL_LINES', 'GL_TRIANGLES', 
# glEnable features
'GL_BLEND', 'GL_POLYGON_OFFSET_FILL',  'GL_POLYGON_OFFSET_LINE',
'GL_CULL_FACE', 'GL_DEPTH_CLAMP', 'GL_DEPTH_TEST', 
'GL_DITHER', 'GL_LINE_SMOOTH', 'GL_MULTISAMPLE',
'GL_SAMPLE_ALPHA_TO_ONE', 'GL_STENCIL_TEST',
# glHints
'GL_PERSPECTIVE_CORRECTION_HINT', 'GL_LINE_SMOOTH_HINT', 
# Texture filtering
'GL_TEXTURE_MAG_FILTER', 'GL_TEXTURE_MIN_FILTER',
# Alpha blending functions
'GL_ZERO', 'GL_ONE', 'GL_SRC_COLOR', 'GL_ONE_MINUS_SRC_COLOR', 
'GL_DST_COLOR', 'GL_ONE_MINUS_DST_COLOR', 'GL_SRC_ALPHA',
'GL_ONE_MINUS_SRC_ALPHA', 'GL_DST_ALPHA', 'GL_ONE_MINUS_DST_ALPHA',
'GL_CONSTANT_COLOR', 'GL_ONE_MINUS_CONSTANT_COLOR', 'GL_CONSTANT_ALPHA',
'GL_ONE_MINUS_CONSTANT_ALPHA', 'GL_SRC_ALPHA_SATURATE', 'GL_SRC1_COLOR',
'GL_ONE_MINUS_SRC1_COLOR', 'GL_SRC1_ALPHA', 'GL_ONE_MINUS_SRC1_ALPHA',
# Misc - well, unsorted.
'GL_ALPHA', 'GL_ARRAY_BUFFER', "GL_BLEND_COLOR",
'GL_COLOR_BUFFER_BIT', 'GL_DEPTH_BUFFER_BIT',   
'GL_FRONT', 
'GL_LUMINANCE', 'GL_LUMINANCE_ALPHA', 'GL_NEAREST', 'GL_NICEST',  

'GL_INFO_LOG_LENGTH', 'GL_COLOR_LOGIC_OP', 'GL_FRAMEBUFFER_SRGB', 
'GL_FRAMEBUFFER_ATTACHMENT_COLOR_ENCODING',  'GL_FILL', 
'GL_LINE', 'GL_POLYGON_OFFSET_POINT', 'GL_SAMPLE_ALPHA_TO_COVERAGE', 
'GL_SAMPLE_COVERAGE', 'GL_PROGRAM_POINT_SIZE',  'GL_RENDERER', 
]

gl3Ints = ['GL_FRAMEBUFFER', 'GL_FRAMEBUFFER_COMPLETE', 
           'GL_MAX_COLOR_ATTACHMENTS']
gl2RenderToTextureInts = ['GL_FRAMEBUFFER_EXT', 'GL_FRAMEBUFFER_COMPLETE_EXT', 
                          'GL_MAX_COLOR_ATTACHMENTS_EXT']

glInts.extend(['GL_TEXTURE%i' % i for i in xrange(32)])
glInts.extend(['GL_COLOR_ATTACHMENT%i' % i for i in xrange(16)])
# Not implemented: 'GL_CLIP_DISTANCEi'
# GL>2.1? 'GL_SCISSOR_TE', GL_SAMP, 'GL_DEBUG_OUTPUT_SYNCHRONOUS', 'GL_POLYGON_SMOO',
# 'GL_SAMPLE_', 'GL_DEBUG_OUTPUT'
glfwInts = ['GLFW_VERSION_MAJOR', 'GLFW_VERSION_MINOR', 
'GLFW_VERSION_REVISION', 'GLFW_RELEASE', 'GLFW_PRESS', 'GLFW_KEY_UNKNOWN', 
'GLFW_KEY_SPACE', 'GLFW_KEY_SPECIAL', 'GLFW_KEY_ESC', 'GLFW_KEY_F1', 
'GLFW_KEY_F2', 'GLFW_KEY_F3', 'GLFW_KEY_F4', 'GLFW_KEY_F5', 'GLFW_KEY_F6', 
'GLFW_KEY_F7', 'GLFW_KEY_F8', 'GLFW_KEY_F9', 'GLFW_KEY_F10', 'GLFW_KEY_F11', 
'GLFW_KEY_F12', 'GLFW_KEY_F13', 'GLFW_KEY_F14', 'GLFW_KEY_F15', 
'GLFW_KEY_F16', 'GLFW_KEY_F17', 'GLFW_KEY_F18', 'GLFW_KEY_F19', 
'GLFW_KEY_F20', 'GLFW_KEY_F21', 'GLFW_KEY_F22', 'GLFW_KEY_F23', 
'GLFW_KEY_F24', 'GLFW_KEY_F25', 'GLFW_KEY_UP', 'GLFW_KEY_DOWN', 
'GLFW_KEY_LEFT', 'GLFW_KEY_RIGHT', 'GLFW_KEY_LSHIFT', 'GLFW_KEY_RSHIFT', 
'GLFW_KEY_LCTRL', 'GLFW_KEY_RCTRL', 'GLFW_KEY_LALT', 'GLFW_KEY_RALT', 
'GLFW_KEY_TAB', 'GLFW_KEY_ENTER', 'GLFW_KEY_BACKSPACE', 'GLFW_KEY_INSERT',
'GLFW_KEY_DEL', 'GLFW_KEY_PAGEUP', 'GLFW_KEY_PAGEDOWN', 'GLFW_KEY_HOME', 
'GLFW_KEY_END', 'GLFW_KEY_KP_0', 'GLFW_KEY_KP_1', 'GLFW_KEY_KP_2', 
'GLFW_KEY_KP_3', 'GLFW_KEY_KP_4', 'GLFW_KEY_KP_5', 'GLFW_KEY_KP_6', 
'GLFW_KEY_KP_7', 'GLFW_KEY_KP_8', 'GLFW_KEY_KP_9', 'GLFW_KEY_KP_DIVIDE',
'GLFW_KEY_KP_MULTIPLY', 'GLFW_KEY_KP_SUBTRACT', 'GLFW_KEY_KP_ADD',
'GLFW_KEY_KP_DECIMAL', 'GLFW_KEY_KP_EQUAL', 'GLFW_KEY_KP_ENTER', 
'GLFW_KEY_KP_NUM_LOCK', 'GLFW_KEY_CAPS_LOCK', 'GLFW_KEY_SCROLL_LOCK', 
'GLFW_KEY_PAUSE', 'GLFW_KEY_LSUPER', 'GLFW_KEY_RSUPER', 'GLFW_KEY_MENU', 
'GLFW_KEY_LAST', 'GLFW_MOUSE_BUTTON_1', 'GLFW_MOUSE_BUTTON_2', 
'GLFW_MOUSE_BUTTON_3', 'GLFW_MOUSE_BUTTON_4', 'GLFW_MOUSE_BUTTON_5', 
'GLFW_MOUSE_BUTTON_6', 'GLFW_MOUSE_BUTTON_7', 'GLFW_MOUSE_BUTTON_8', 
'GLFW_MOUSE_BUTTON_LAST', 'GLFW_MOUSE_BUTTON_LEFT', 'GLFW_MOUSE_BUTTON_RIGHT',
'GLFW_MOUSE_BUTTON_MIDDLE', 'GLFW_JOYSTICK_1', 'GLFW_JOYSTICK_2', 
'GLFW_JOYSTICK_3', 'GLFW_JOYSTICK_4', 'GLFW_JOYSTICK_5', 'GLFW_JOYSTICK_6', 
'GLFW_JOYSTICK_7', 'GLFW_JOYSTICK_8', 'GLFW_JOYSTICK_9', 'GLFW_JOYSTICK_10', 
'GLFW_JOYSTICK_11', 'GLFW_JOYSTICK_12', 'GLFW_JOYSTICK_13', 
'GLFW_JOYSTICK_14', 'GLFW_JOYSTICK_15', 'GLFW_JOYSTICK_16', 
'GLFW_JOYSTICK_LAST', 'GLFW_WINDOW', 'GLFW_FULLSCREEN', 'GLFW_OPENED', 
'GLFW_ACTIVE', 'GLFW_ICONIFIED', 'GLFW_ACCELERATED', 'GLFW_RED_BITS', 
'GLFW_GREEN_BITS', 'GLFW_BLUE_BITS', 'GLFW_ALPHA_BITS', 'GLFW_DEPTH_BITS', 
'GLFW_STENCIL_BITS', 'GLFW_REFRESH_RATE', 'GLFW_ACCUM_RED_BITS', 
'GLFW_ACCUM_GREEN_BITS', 'GLFW_ACCUM_BLUE_BITS', 'GLFW_ACCUM_ALPHA_BITS', 
'GLFW_AUX_BUFFERS', 'GLFW_STEREO', 'GLFW_WINDOW_NO_RESIZE', 
'GLFW_FSAA_SAMPLES', 'GLFW_OPENGL_VERSION_MAJOR', 'GLFW_OPENGL_VERSION_MINOR', 
'GLFW_OPENGL_FORWARD_COMPAT', 'GLFW_OPENGL_DEBUG_CONTEXT', 
'GLFW_OPENGL_PROFILE', 'GLFW_OPENGL_CORE_PROFILE', 'GLFW_OPENGL_COMPAT_PROFILE', 
'GLFW_MOUSE_CURSOR', 'GLFW_STICKY_KEYS', 'GLFW_STICKY_MOUSE_BUTTONS', 
'GLFW_SYSTEM_KEYS', 'GLFW_KEY_REPEAT', 'GLFW_AUTO_POLL_EVENTS', 'GLFW_WAIT', 
'GLFW_NOWAIT', 'GLFW_PRESENT', 'GLFW_AXES', 'GLFW_BUTTONS', 
'GLFW_NO_RESCALE_BIT', 'GLFW_ORIGIN_UL_BIT', 'GLFW_BUILD_MIPMAPS_BIT', 
'GLFW_ALPHA_MAP_BIT', 'GLFW_INFINITY']

glewInts = ['GLEW_OK']

typedefs = """
typedef unsigned int    GLenum;
typedef unsigned char    GLboolean;
typedef char    GLchar;
typedef unsigned int    GLbitfield;
typedef void        GLvoid;
typedef signed char    GLbyte;        /* 1-byte signed */
typedef short        GLshort;    /* 2-byte signed */
typedef int        GLint;        /* 4-byte signed */
typedef unsigned char    GLubyte;    /* 1-byte unsigned */
typedef unsigned short    GLushort;    /* 2-byte unsigned */
typedef unsigned int    GLuint;        /* 4-byte unsigned */
typedef int        GLsizei;    /* 4-byte signed */
typedef float        GLfloat;    /* single precision float */
typedef float        GLclampf;    /* single precision float in [0,1] */
typedef double        GLdouble;    /* double precision float */
typedef double        GLclampd;    /* double precision float in [0,1] */
// This needs checking
typedef int GLsizeiptr;
typedef int GLintptr;
"""
includes = """
#include <GL/glew.h>
#include <stdio.h>
#include <GL/glfw.h>
"""

#class NoisyDebug(type):
#    def __init__(cls, *args, **kwargs):
#        super(NoisyDebug, cls).__init__(*args, **kwargs)
#        for x in dir(cls):
#            if callable(getattr(cls, x)) and not x.startswith('_'):
#                cls.makeNoisy(x)
#                
#    def __getattribute__(cls, key):
#        print 'Get raw %s' % key
#        return super(NoisyDebug, cls).__getattribute__(key)
#    
#    def makeNoisy(cls, x):
#        attr = getattr(cls, x)
#        def wrap(self, *args, **kwargs):
#            print 'Raw call %s' % x
#            attr(self, *args, **kwargs)
#        setattr(cls, x, wrap)
         
class RawBinding(object):
    """An object that contains the raw, unusable GL binding - or at least, those it 
    knows about"""
    #def __getattribute__(self, key):
    #    print 'Raw %s' % key
    #    return super(RawBinding, self).__getattribute__(key)
    
    def __init__(self, listOfIntConstants):
        directory = os.path.dirname(__file__)
        with open(os.path.join(directory, 'cffigl.h'), 'r') as f:
            cdefs = f.read()

        ccode = includes
            
        listOfIntConstants = set(listOfIntConstants)
        self.constNames = listOfIntConstants
        cdefs += '\n'.join(('const GLint py%s;' % name 
                            for name in listOfIntConstants))
        ccode += '\n'.join(('const GLint py%s = %s;' % (name, name) 
                              for name in listOfIntConstants))
        self.ffi = FFI()
        self.ffi.cdef(cdefs)
        self.C = self.ffi.verify(ccode, libraries=['GLEW', 'glfw'])
        for name in self.C.__dict__:
            if not name.startswith('_'):
                if name.startswith('py'):
                    rname = name[2:]
                else:
                    rname = name
                setattr(self, rname, getattr(self.C, name))     
                
    def __getitem__(self, key):
        if key in self.__dict__:
            return self.__dict__[key]
        
    #def __getattribute__(self, key):
    #    thing = super(RawBinding, self).__getattribute__(key)
    #    if callable(thing):
    #        def wrap(*args, **kwargs):
    #            print key
    #            return thing(*args, **kwargs)
    #        return wrap
    #    return thing
                
                
            
if __name__ == '__main__':
    test = RawBinding(glInts + glfwInts + glewInts)
        