"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.glcommon.glfwcommon

A GLCommon instance that uses glfw. Requires cffigl binding.
"""

from pygl3display.glcommon.base import GLCommonClass
from pygl3display.gl.cffigl import (glfwInit, glfwOpenWindowHint, 
    glfwOpenWindow, glfwTerminate, glfwSwapInterval, glfwSwapBuffers,
    GLFW_FSAA_SAMPLES, GLFW_OPENGL_VERSION_MAJOR, GLFW_OPENGL_VERSION_MINOR,
    GLFW_WINDOW, GLFW_FULLSCREEN) 

class GLCommonGLFW(GLCommonClass):
    """Backend functions for GLFW"""
    def init(self):
        """Initialise GLFW"""
        if not glfwInit():
            raise Exception('GLFW Initialisation failed')
        
    def createContext(self, resolution, doublebuffer, fullscreen):
        """Create a GLFW Context"""
        if not glfwInit():
            raise Exception('GLFW Initialisation failed')
        glfwOpenWindowHint(GLFW_FSAA_SAMPLES, 4)
        glfwOpenWindowHint(GLFW_OPENGL_VERSION_MAJOR, 2)
        glfwOpenWindowHint(GLFW_OPENGL_VERSION_MINOR, 1)
        mode = GLFW_WINDOW if not fullscreen else GLFW_FULLSCREEN
    
        if not glfwOpenWindow(resolution[0], resolution[1], 0, 0, 0, 0, 
                              32, 0, mode):
            glfwTerminate()
            raise Exception('Could not open GLFW Window')
        resyncInterval = 1 if doublebuffer else 0
        glfwSwapInterval(resyncInterval)
        
    redraw = staticmethod(glfwSwapBuffers)
        

