"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.hacks

Applies hacks to get PyGL3Display to run on known bad drivers.
applyHacks is called automatically on glInitEnvironment
"""

from pygl3display.gl import glGetString, GL_RENDERER
from pygl3display.initialise import Initialiser

def applyHacks():
    """Applies hacks to PyGL3Display to allow it to work on bad drivers,
    whilst not impairing performance on good drivers.
    Current bad driver list:
        Virtualbox "Chromium" driver
    """
    renderer = glGetString(GL_RENDERER)
    if renderer == u'Chromium':
        # Virtualboxs Chromium driver allows the creation of large textures
        # that it can't support , causing crashes.
        # Compensate by reducing max texture size to 1024
        from pygl3display.texture.twod import TwoDTextureCommon
        from pygl3display.texture.oned import OneDTextureCommon
        TwoDTextureCommon.maxTextureSize = 1024
        OneDTextureCommon.maxTextureSize = 1024
        
Initialiser.addFunction('applyHacks', applyHacks, {})