"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3).

pygl3display.initialise

Used to defer the execution of some initialisation until after a GL
context has been set up.
"""

class InitialiserClass(object):
    """Used to defer some initialisation until after a GL context is ready"""
    def __init__(self):
        """Initial setup"""
        self.funcs = {}

    def addFunction(self, name, func, defaultParams):
        """Add a function and default parameters"""
        self.funcs[name] = (func, defaultParams)
    
    def initialise(self):
        """Run all functions that have been set up"""
        for func, params in self.funcs.values():
            func(**params)
        
Initialiser = InitialiserClass()