"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.sprites.pypngimages

This file provides functions that let PyGL3Display use PyPNG for decoding 
images
"""

from png import Reader

from pygl3display.sprites.image import Surface
from pygl3display.sprites.rawimage import RawImage

class PNGRawImage(RawImage):
    """A wrapper object to load raw data from a PNG file"""
    def __init__(self, fn):
        """Load data from the given file name"""
        reader = Reader(filename=fn)
        width, height, pixels, meta = reader.read_flat()
        self.size = (width, height)
        self.raw = list(pixels)
        
class PNGSurface(Surface):
    """A wrapper object to create a surface from a the given PNG filename"""
    def __init__(self, fn):
        """Setup a surface from the filename"""
        self.rawimage = PNGRawImage(fn)
        size = self.rawimage.getSize()
        Surface.__init__(self, size)
        self.upload(self.rawimage)

def loadImage(imagefn):
    """Load a PNG image to a new surface"""
    return PNGSurface(imagefn)
        
