"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.sprites.rawimage

Provides RawImage, the wrapper around images data before it is uploaded to the
GPU
"""

class RawImage(object):
    """A RawImage is an image before it has been uploaded. It provides
    information on the size of the image and a raw form of the image. This API
    should be replicated by the backends for their raw image representation"""
    def __init__(self, size, raw):
        """Initialise a raw image"""
        self.size = size
        self.raw = raw
        
    def getSize(self):
        """Return the size of the image"""
        return self.size
        
    def getSizeWrapped(self):
        """Return the size necessary to accommodate a wrapped image"""
        size = self.getSize()
        return (size[0] + 2, size[1] + 2)
        
    def getRaw(self):
        """Return an RGBA raw of the image"""
        return self.raw
        
    def getRawWrapped(self):
        """Return an RGBA raw of the image that is wrapped"""
        raw = self.getRaw()
        xsz, ysz = self.getSize()
        wrapRaw = []
        rowLength = xsz * 4
        for iterable in (ysz-1,), xrange(ysz), (0,):
            for x in iterable:
                base = xsz * x * 4
                wrapRaw.extend((raw[base+rowLength-4:base+rowLength], 
                                raw[base:base+rowLength], raw[base:base+4]))
        return ''.join(wrapRaw)
