"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.sprites.spritecommon

Provides some utility classes and management of defaults for the module 
"""

from __future__ import division

from collections import namedtuple

from pygl3display.vertexarrays import VertexArraySpec
from pygl3display.gl import GL_TRUE, GL_UNSIGNED_BYTE, cdata
from pygl3display.twodbases import TwoDBaseCommonClass

GL3Rect = namedtuple('GL3Rect', ('x', 'y', 'width', 'height', 'area'))

#GL3RectNA = namedtuple('GL3RectNA', ('x', 'y', 'width', 'height', 'area'))
class GL3RectNA(GL3Rect):
    """These are identical to GL3Rects, but the class marks something
    that has *not* been allocated by a texture atlas, and hence should
    not be remembered when its released.""" 

class SpriteCommonClass(TwoDBaseCommonClass):
    """The shared state between 2d sprites
    Variables:
      spec: a vertex array specification for sprite shaders
    """
    def __init__(self):
        """Initialise the Sprite Common"""
        self.defaultShaderClass = None
        self.defaultAtlasClass = None

    spec = VertexArraySpec('TwoDSpritesArray', slotMultiplier=1.5)
    spec.addArray('pos')
    spec.addArray('color', cType=cdata['B'], 
                  glType=GL_UNSIGNED_BYTE, glNorm=GL_TRUE)
    spec.addArray('cameras1')
    spec.addArray('cameras2')
    spec.addArray('fragCameras')
    spec.addArray('texCoords', dynamic=False)
    spec.addArray('texRatios', dynamic=False)
    spec.addArray('rotozoom')
    spec.addArray('filters')
    spec.addAttrib('pos', 'Position', 2, 0)
    spec.addAttrib('pos', 'Offset', 2, 2, True)
    spec.addAttrib('texCoords', 'TexCoord', 4, 0)
    spec.addAttrib('texRatios', 'TexRatio', 2, 0, True)
    spec.addAttrib('color', 'Color', 4, 0)
    spec.addAttrib('color', 'Colors', 4, 0, True)
    spec.addAttrib('cameras1', 'Camera1', 1, None)
    spec.addAttrib('cameras2', 'Camera2', 1, None)
    spec.addAttrib('fragCameras', 'FragCamera', 1, None)
    spec.addAttrib('rotozoom', 'ScaleX', 1, 0)
    spec.addAttrib('rotozoom', 'ScaleY', 1, 1)
    spec.addAttrib('rotozoom', 'Rotation', 1, 2)
    spec.addAttrib('filters', 'FilterType', 1, 0)
    
    def setDefaultShaderClass(self, cls):
        """Set the default shader class to use"""
        self.defaultShaderClass = cls
        
    def setDefaultAtlasClass(self, cls):
        """Set the default atlas class to use"""
        self.defaultAtlasClass = cls
        
    def makeDefaultShader(self):
        """Create the default shader object"""
        self.defaultShader = self.defaultShaderClass(self.defaultAtlasClass)
            
    def setDefaultFilter(self, filter):
        """Set default filter for the the default shader"""
        self.defaultShader.setDefaultFilter(filter)

SpriteCommon = SpriteCommonClass()
