"""
PyGL3Display
(C) David Griffin (2010-present)
"Habilain" is a pseudonym used by David Griffin in a number of places.

A high performance OpenGL library

Licensed under GPLv2 or GPLv3. You should have received a copy of both of these
licenses in the files GPLv2.txt and GPLv3.txt. If not these licenses can be
found at http://www.gnu.org/licenses/gpl-2.0.html (GPLv2) and
http://www.gnu.org/licenses/gpl-3.0.html (GPLv3). 

pygl3display.uniforms.store

Provides a uniform store for shaders to use
"""

import weakref

from pygl3display.glcommoninst import GLCommon

class UniformStore(object):
    """A UniformStore handles the storing and binding of Uniforms for a 
    shader"""
    def __init__(self, uniforms=None):
        """Initalise the uniform store.
        Args: 
          uniforms: A set of Uniform objects
        """
        self.uniforms = uniforms if uniforms is not None else set()
        self.shaderBindArgs = weakref.WeakKeyDictionary()
             
    def bind(self):
        """Bind the uniforms in this store"""
        shader = GLCommon.boundShader
            
        boundUniformStore = GLCommon.boundUniformStore
        if (shader not in boundUniformStore or 
           boundUniformStore[shader] is not self):
            boundUniformStore[shader] = self
            force = True
        else:
            force = False
        for uniform in self.uniforms:
            uniform.bind(shader, force)
                    
    def update(self, uniforms):
        """Update uniforms set"""
        self.uniforms.update(uniforms)